@@ -19,6 +19,7 @@ import contextlib
 import itertools
 import threading
 
+from automaton import runners
 from concurrent import futures
 import fasteners
 import networkx as nx
@@ -26,6 +27,7 @@ from oslo_utils import excutils
 from oslo_utils import strutils
 import six
 
+from taskflow.engines.action_engine import builder
 from taskflow.engines.action_engine import compiler
 from taskflow.engines.action_engine import executor
 from taskflow.engines.action_engine import runtime
@@ -54,20 +56,80 @@ def _start_stop(task_executor, retry_executor):
         task_executor.stop()
 
 
+def _pre_check(check_compiled=True, check_storage_ensured=True,
+               check_validated=True):
+    """Engine state precondition checking decorator."""
+
+    def decorator(meth):
+        do_what = meth.__name__
+
+        @six.wraps(meth)
+        def wrapper(self, *args, **kwargs):
+            if check_compiled and not self._compiled:
+                raise exc.InvalidState("Can not %s an engine which"
+                                       " has not been compiled" % do_what)
+            if check_storage_ensured and not self._storage_ensured:
+                raise exc.InvalidState("Can not %s an engine"
+                                       " which has not had its storage"
+                                       " populated" % do_what)
+            if check_validated and not self._validated:
+                raise exc.InvalidState("Can not %s an engine which"
+                                       " has not been validated" % do_what)
+            return meth(self, *args, **kwargs)
+
+        return wrapper
+
+    return decorator
+
+
 class ActionEngine(base.Engine):
     """Generic action-based engine.
 
     This engine compiles the flow (and any subflows) into a compilation unit
     which contains the full runtime definition to be executed and then uses
-    this compilation unit in combination with the executor, runtime, runner
-    and storage classes to attempt to run your flow (and any subflows &
-    contained atoms) to completion.
+    this compilation unit in combination with the executor, runtime, machine
+    builder and storage classes to attempt to run your flow (and any
+    subflows & contained atoms) to completion.
 
     NOTE(harlowja): during this process it is permissible and valid to have a
     task or multiple tasks in the execution graph fail (at the same time even),
     which will cause the process of reversion or retrying to commence. See the
     valid states in the states module to learn more about what other states
     the tasks and flow being ran can go through.
+
+    **Engine options:**
+
+    +----------------------+-----------------------+------+------------+
+    | Name/key             | Description           | Type | Default    |
+    +======================+=======================+======+============+
+    | ``defer_reverts``    | This option lets you  | bool | ``False``  |
+    |                      | safely nest flows     |      |            |
+    |                      | with retries inside   |      |            |
+    |                      | flows without retries |      |            |
+    |                      | and it still behaves  |      |            |
+    |                      | as a user would       |      |            |
+    |                      | expect (for example   |      |            |
+    |                      | if the retry gets     |      |            |
+    |                      | exhausted it reverts  |      |            |
+    |                      | the outer flow unless |      |            |
+    |                      | the outer flow has a  |      |            |
+    |                      | has a separate retry  |      |            |
+    |                      | behavior).            |      |            |
+    +----------------------+-----------------------+------+------------+
+    | ``inject_transient`` | When true, values     | bool | ``True``   |
+    |                      | that are local to     |      |            |
+    |                      | each atoms scope      |      |            |
+    |                      | are injected into     |      |            |
+    |                      | storage into a        |      |            |
+    |                      | transient location    |      |            |
+    |                      | (typically a local    |      |            |
+    |                      | dictionary), when     |      |            |
+    |                      | false those values    |      |            |
+    |                      | are instead persisted |      |            |
+    |                      | into atom details     |      |            |
+    |                      | (and saved in a non-  |      |            |
+    |                      | transient manner).    |      |            |
+    +----------------------+-----------------------+------+------------+
     """
 
     NO_RERAISING_STATES = frozenset([states.SUSPENDED, states.SUCCESS])
@@ -77,6 +139,21 @@ class ActionEngine(base.Engine):
     failure/s that were captured (if any) to get reraised.
     """
 
+    IGNORABLE_STATES = frozenset(
+        itertools.chain([states.SCHEDULING, states.WAITING, states.RESUMING,
+                         states.ANALYZING], builder.META_STATES))
+    """
+    Informational states this engines internal machine yields back while
+    running, not useful to have the engine record but useful to provide to
+    end-users when doing execution iterations via :py:meth:`.run_iter`.
+    """
+
+    MAX_MACHINE_STATES_RETAINED = 10
+    """
+    During :py:meth:`~.run_iter` the last X state machine transitions will
+    be recorded (typically only useful on failure).
+    """
+
     def __init__(self, flow, flow_detail, backend, options):
         super(ActionEngine, self).__init__(flow, flow_detail, backend, options)
         self._runtime = None
@@ -86,22 +163,21 @@ class ActionEngine(base.Engine):
         self._lock = threading.RLock()
         self._state_lock = threading.RLock()
         self._storage_ensured = False
+        self._validated = False
         # Retries are not *currently* executed out of the engines process
         # or thread (this could change in the future if we desire it to).
         self._retry_executor = executor.SerialRetryExecutor()
+        self._inject_transient = strutils.bool_from_string(
+            self._options.get('inject_transient', True))
 
-    def _check(self, name, check_compiled, check_storage_ensured):
-        """Check (and raise) if the engine has not reached a certain stage."""
-        if check_compiled and not self._compiled:
-            raise exc.InvalidState("Can not %s an engine which"
-                                   " has not been compiled" % name)
-        if check_storage_ensured and not self._storage_ensured:
-            raise exc.InvalidState("Can not %s an engine"
-                                   " which has not has its storage"
-                                   " populated" % name)
-
+    @_pre_check(check_compiled=True,
+                # NOTE(harlowja): We can alter the state of the
+                # flow without ensuring its storage is setup for
+                # its atoms (since this state change does not affect
+                # those units).
+                check_storage_ensured=False,
+                check_validated=False)
     def suspend(self):
-        self._check('suspend', True, False)
         self._change_state(states.SUSPENDING)
 
     @property
@@ -140,31 +216,37 @@ class ActionEngine(base.Engine):
                                backend=self._backend,
                                scope_fetcher=_scope_fetcher)
 
-    def run(self):
+    def run(self, timeout=None):
+        """Runs the engine (or die trying).
+
+        :param timeout: timeout to wait for any atoms to complete (this timeout
+            will be used during the waiting period that occurs when
+            unfinished atoms are being waited on).
+        """
         with fasteners.try_lock(self._lock) as was_locked:
             if not was_locked:
                 raise exc.ExecutionFailure("Engine currently locked, please"
                                            " try again later")
-            for _state in self.run_iter():
+            for _state in self.run_iter(timeout=timeout):
                 pass
 
     def run_iter(self, timeout=None):
         """Runs the engine using iteration (or die trying).
 
-        :param timeout: timeout to wait for any tasks to complete (this timeout
+        :param timeout: timeout to wait for any atoms to complete (this timeout
             will be used during the waiting period that occurs after the
-            waiting state is yielded when unfinished tasks are being waited
-            for).
+            waiting state is yielded when unfinished atoms are being waited
+            on).
 
         Instead of running to completion in a blocking manner, this will
         return a generator which will yield back the various states that the
         engine is going through (and can be used to run multiple engines at
-        once using a generator per engine). the iterator returned also
-        responds to the send() method from pep-0342 and will attempt to suspend
-        itself if a truthy value is sent in (the suspend may be delayed until
-        all active tasks have finished).
+        once using a generator per engine). The iterator returned also
+        responds to the ``send()`` method from :pep:`0342` and will attempt to
+        suspend itself if a truthy value is sent in (the suspend may be
+        delayed until all active atoms have finished).
 
-        NOTE(harlowja): using the run_iter method will **not** retain the
+        NOTE(harlowja): using the ``run_iter`` method will **not** retain the
         engine lock while executing so the user should ensure that there is
         only one entity using a returned engine iterator (one per engine) at a
         given time.
@@ -172,19 +254,29 @@ class ActionEngine(base.Engine):
         self.compile()
         self.prepare()
         self.validate()
-        runner = self._runtime.runner
-        last_state = None
+        # Keep track of the last X state changes, which if a failure happens
+        # are quite useful to log (and the performance of tracking this
+        # should be negligible).
+        last_transitions = collections.deque(
+            maxlen=max(1, self.MAX_MACHINE_STATES_RETAINED))
         with _start_stop(self._task_executor, self._retry_executor):
             self._change_state(states.RUNNING)
             try:
                 closed = False
-                for (last_state, failures) in runner.run_iter(timeout=timeout):
-                    if failures:
-                        failure.Failure.reraise_if_any(failures)
+                machine, memory = self._runtime.builder.build(timeout=timeout)
+                r = runners.FiniteRunner(machine)
+                for transition in r.run_iter(builder.START):
+                    last_transitions.append(transition)
+                    _prior_state, new_state = transition
+                    # NOTE(harlowja): skip over meta-states
+                    if new_state in builder.META_STATES:
+                        continue
+                    if new_state == states.FAILURE:
+                        failure.Failure.reraise_if_any(memory.failures)
                     if closed:
                         continue
                     try:
-                        try_suspend = yield last_state
+                        try_suspend = yield new_state
                     except GeneratorExit:
                         # The generator was closed, attempt to suspend and
                         # continue looping until we have cleanly closed up
@@ -196,16 +288,42 @@ class ActionEngine(base.Engine):
                             self.suspend()
             except Exception:
                 with excutils.save_and_reraise_exception():
+                    LOG.exception("Engine execution has failed, something"
+                                  " bad must of happened (last"
+                                  " %s machine transitions were %s)",
+                                  last_transitions.maxlen,
+                                  list(last_transitions))
                     self._change_state(states.FAILURE)
             else:
-                ignorable_states = getattr(runner, 'ignorable_states', [])
-                if last_state and last_state not in ignorable_states:
-                    self._change_state(last_state)
-                    if last_state not in self.NO_RERAISING_STATES:
-                        it = itertools.chain(
-                            six.itervalues(self.storage.get_failures()),
-                            six.itervalues(self.storage.get_revert_failures()))
-                        failure.Failure.reraise_if_any(it)
+                if last_transitions:
+                    _prior_state, new_state = last_transitions[-1]
+                    if new_state not in self.IGNORABLE_STATES:
+                        self._change_state(new_state)
+                        if new_state not in self.NO_RERAISING_STATES:
+                            failures = self.storage.get_failures()
+                            more_failures = self.storage.get_revert_failures()
+                            fails = itertools.chain(
+                                six.itervalues(failures),
+                                six.itervalues(more_failures))
+                            failure.Failure.reraise_if_any(fails)
+
+    @staticmethod
+    def _check_compilation(compilation):
+        """Performs post compilation validation/checks."""
+        seen = set()
+        dups = set()
+        execution_graph = compilation.execution_graph
+        for node, node_attrs in execution_graph.nodes_iter(data=True):
+            if node_attrs['kind'] in compiler.ATOMS:
+                atom_name = node.name
+                if atom_name in seen:
+                    dups.add(atom_name)
+                else:
+                    seen.add(atom_name)
+        if dups:
+            raise exc.Duplicate(
+                "Atoms with duplicate names found: %s" % (sorted(dups)))
+        return compilation
 
     def _change_state(self, state):
         with self._state_lock:
@@ -223,30 +341,27 @@ class ActionEngine(base.Engine):
 
     def _ensure_storage(self):
         """Ensure all contained atoms exist in the storage unit."""
-        transient = strutils.bool_from_string(
-            self._options.get('inject_transient', True))
         self.storage.ensure_atoms(
-            self._compilation.execution_graph.nodes_iter())
-        for node in self._compilation.execution_graph.nodes_iter():
-            if node.inject:
-                self.storage.inject_atom_args(node.name,
-                                              node.inject,
-                                              transient=transient)
+            self._runtime.analyzer.iterate_nodes(compiler.ATOMS))
+        for atom in self._runtime.analyzer.iterate_nodes(compiler.ATOMS):
+            if atom.inject:
+                self.storage.inject_atom_args(atom.name, atom.inject,
+                                              transient=self._inject_transient)
 
     @fasteners.locked
+    @_pre_check(check_validated=False)
     def validate(self):
-        self._check('validate', True, True)
         # At this point we can check to ensure all dependencies are either
         # flow/task provided or storage provided, if there are still missing
         # dependencies then this flow will fail at runtime (which we can avoid
         # by failing at validation time).
-        execution_graph = self._compilation.execution_graph
-        if LOG.isEnabledFor(logging.BLATHER):
-            LOG.blather("Validating scoping and argument visibility for"
-                        " execution graph with %s nodes and %s edges with"
-                        " density %0.3f", execution_graph.number_of_nodes(),
-                        execution_graph.number_of_edges(),
-                        nx.density(execution_graph))
+        if LOG.isEnabledFor(logging.TRACE):
+            execution_graph = self._compilation.execution_graph
+            LOG.trace("Validating scoping and argument visibility for"
+                      " execution graph with %s nodes and %s edges with"
+                      " density %0.3f", execution_graph.number_of_nodes(),
+                      execution_graph.number_of_edges(),
+                      nx.density(execution_graph))
         missing = set()
         # Attempt to retain a chain of what was missing (so that the final
         # raised exception for the flow has the nodes that had missing
@@ -254,18 +369,17 @@ class ActionEngine(base.Engine):
         last_cause = None
         last_node = None
         missing_nodes = 0
-        fetch_func = self.storage.fetch_unsatisfied_args
-        for node in execution_graph.nodes_iter():
-            node_missing = fetch_func(node.name, node.rebind,
-                                      optional_args=node.optional)
-            if node_missing:
-                cause = exc.MissingDependencies(node,
-                                                sorted(node_missing),
+        for atom in self._runtime.analyzer.iterate_nodes(compiler.ATOMS):
+            atom_missing = self.storage.fetch_unsatisfied_args(
+                atom.name, atom.rebind, optional_args=atom.optional)
+            if atom_missing:
+                cause = exc.MissingDependencies(atom,
+                                                sorted(atom_missing),
                                                 cause=last_cause)
                 last_cause = cause
-                last_node = node
+                last_node = atom
                 missing_nodes += 1
-                missing.update(node_missing)
+                missing.update(atom_missing)
         if missing:
             # For when a task is provided (instead of a flow) and that
             # task is the only item in the graph and its missing deps, avoid
@@ -276,10 +390,11 @@ class ActionEngine(base.Engine):
                 raise exc.MissingDependencies(self._flow,
                                               sorted(missing),
                                               cause=last_cause)
+        self._validated = True
 
     @fasteners.locked
+    @_pre_check(check_storage_ensured=False, check_validated=False)
     def prepare(self):
-        self._check('prepare', True, False)
         if not self._storage_ensured:
             # Set our own state to resuming -> (ensure atoms exist
             # in storage) -> suspended in the storage unit and notify any
@@ -293,11 +408,8 @@ class ActionEngine(base.Engine):
             self.reset()
 
     @fasteners.locked
+    @_pre_check(check_validated=False)
     def reset(self):
-        if not self._storage_ensured:
-            raise exc.InvalidState("Can not reset an engine"
-                                   " which has not has its storage"
-                                   " populated")
         # This transitions *all* contained atoms back into the PENDING state
         # with an intention to EXECUTE (or dies trying to do that) and then
         # changes the state of the flow to PENDING so that it can then run...
@@ -308,12 +420,13 @@ class ActionEngine(base.Engine):
     def compile(self):
         if self._compiled:
             return
-        self._compilation = self._compiler.compile()
+        self._compilation = self._check_compilation(self._compiler.compile())
         self._runtime = runtime.Runtime(self._compilation,
                                         self.storage,
                                         self.atom_notifier,
                                         self._task_executor,
-                                        self._retry_executor)
+                                        self._retry_executor,
+                                        options=self._options)
         self._runtime.compile()
         self._compiled = True
 
@@ -342,7 +455,7 @@ class _ExecutorTextMatch(collections.namedtuple('_ExecutorTextMatch',
 class ParallelActionEngine(ActionEngine):
     """Engine that runs tasks in parallel manner.
 
-    Supported option keys:
+    **Additional engine options:**
 
     * ``executor``: a object that implements a :pep:`3148` compatible executor
       interface; it will be used for scheduling tasks. The following
@@ -372,6 +485,19 @@ String (case insensitive)    Executor used
 ``threads``                  :class:`~.executor.ParallelThreadTaskExecutor`
 ===========================  ===============================================
 
+    * ``max_workers``: a integer that will affect the number of parallel
+      workers that are used to dispatch tasks into (this number is bounded
+      by the maximum parallelization your workflow can support).
+
+    * ``dispatch_periodicity``: a float (in seconds) that will affect the
+      parallel process task executor (and therefore is **only** applicable when
+      the executor provided above is of the process variant). This number
+      affects how much time the process task executor waits for messages from
+      child processes (typically indicating they have finished or failed). A
+      lower number will have high granularity but *currently* involves more
+      polling while a higher number will involve less polling but a slower time
+      for an engine to notice a task has completed.
+
     .. |cfp| replace:: concurrent.futures.process
     .. |cft| replace:: concurrent.futures.thread
     .. |cf| replace:: concurrent.futures
@@ -450,11 +576,12 @@ String (case insensitive)    Executor used
             else:
                 executor_cls = matched_executor_cls
                 kwargs['executor'] = desired_executor
-        for k in getattr(executor_cls, 'OPTIONS', []):
-            if k == 'executor':
-                continue
-            try:
-                kwargs[k] = options[k]
-            except KeyError:
-                pass
+        try:
+            for (k, value_converter) in executor_cls.constructor_options:
+                try:
+                    kwargs[k] = value_converter(options[k])
+                except KeyError:
+                    pass
+        except AttributeError:
+            pass
         return executor_cls(**kwargs)
