@@ -14,21 +14,31 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections
 import functools
 
+from futurist import waiters
+
+from taskflow import deciders as de
 from taskflow.engines.action_engine.actions import retry as ra
 from taskflow.engines.action_engine.actions import task as ta
 from taskflow.engines.action_engine import analyzer as an
+from taskflow.engines.action_engine import builder as bu
+from taskflow.engines.action_engine import compiler as com
 from taskflow.engines.action_engine import completer as co
-from taskflow.engines.action_engine import runner as ru
 from taskflow.engines.action_engine import scheduler as sched
 from taskflow.engines.action_engine import scopes as sc
-from taskflow import flow as flow_type
+from taskflow.engines.action_engine import traversal as tr
+from taskflow import exceptions as exc
 from taskflow import states as st
-from taskflow import task
-from taskflow.utils import async_utils
 from taskflow.utils import misc
 
+from taskflow.flow import (LINK_DECIDER, LINK_DECIDER_DEPTH)  # noqa
+
+# Small helper to make the edge decider tuples more easily useable...
+_EdgeDecider = collections.namedtuple('_EdgeDecider',
+                                      'from_node,kind,decider,depth')
+
 
 class Runtime(object):
     """A aggregate of runtime objects, properties, ... used during execution.
@@ -39,14 +49,47 @@ class Runtime(object):
     """
 
     def __init__(self, compilation, storage, atom_notifier,
-                 task_executor, retry_executor):
+                 task_executor, retry_executor,
+                 options=None):
         self._atom_notifier = atom_notifier
         self._task_executor = task_executor
         self._retry_executor = retry_executor
         self._storage = storage
         self._compilation = compilation
         self._atom_cache = {}
-        self._atoms_by_kind = {}
+        self._options = misc.safe_copy_dict(options)
+
+    def _walk_edge_deciders(self, graph, atom):
+        """Iterates through all nodes, deciders that alter atoms execution."""
+        # This is basically a reverse breadth first exploration, with
+        # special logic to further traverse down flow nodes as needed...
+        predecessors_iter = graph.predecessors_iter
+        nodes = collections.deque((u_node, atom)
+                                  for u_node in predecessors_iter(atom))
+        visited = set()
+        while nodes:
+            u_node, v_node = nodes.popleft()
+            u_node_kind = graph.node[u_node]['kind']
+            u_v_data = graph.adj[u_node][v_node]
+            try:
+                decider = u_v_data[LINK_DECIDER]
+                decider_depth = u_v_data.get(LINK_DECIDER_DEPTH)
+                if decider_depth is None:
+                    decider_depth = de.Depth.ALL
+                yield _EdgeDecider(u_node, u_node_kind,
+                                   decider, decider_depth)
+            except KeyError:
+                pass
+            if u_node_kind == com.FLOW and u_node not in visited:
+                # Avoid re-exploring the same flow if we get to this same
+                # flow by a different *future* path...
+                visited.add(u_node)
+                # Since we *currently* jump over flow node(s), we need to make
+                # sure that any prior decider that was directed at this flow
+                # node also gets used during future decisions about this
+                # atom node.
+                nodes.extend((u_u_node, u_node)
+                             for u_u_node in predecessors_iter(u_node))
 
     def compile(self):
         """Compiles & caches frequently used execution helper objects.
@@ -58,47 +101,49 @@ class Runtime(object):
         specific scheduler and so-on).
         """
         change_state_handlers = {
-            'task': functools.partial(self.task_action.change_state,
-                                      progress=0.0),
-            'retry': self.retry_action.change_state,
+            com.TASK: functools.partial(self.task_action.change_state,
+                                        progress=0.0),
+            com.RETRY: self.retry_action.change_state,
         }
         schedulers = {
-            'retry': self.retry_scheduler,
-            'task': self.task_scheduler,
+            com.RETRY: self.retry_scheduler,
+            com.TASK: self.task_scheduler,
         }
-        execution_graph = self._compilation.execution_graph
-        all_retry_atoms = []
-        all_task_atoms = []
-        for atom in self.analyzer.iterate_all_nodes():
-            metadata = {}
-            walker = sc.ScopeWalker(self.compilation, atom, names_only=True)
-            if isinstance(atom, task.BaseTask):
-                check_transition_handler = st.check_task_transition
-                change_state_handler = change_state_handlers['task']
-                scheduler = schedulers['task']
-                all_task_atoms.append(atom)
+        check_transition_handlers = {
+            com.TASK: st.check_task_transition,
+            com.RETRY: st.check_retry_transition,
+        }
+        actions = {
+            com.TASK: self.task_action,
+            com.RETRY: self.retry_action,
+        }
+        graph = self._compilation.execution_graph
+        for node, node_data in graph.nodes_iter(data=True):
+            node_kind = node_data['kind']
+            if node_kind in com.FLOWS:
+                continue
+            elif node_kind in com.ATOMS:
+                check_transition_handler = check_transition_handlers[node_kind]
+                change_state_handler = change_state_handlers[node_kind]
+                scheduler = schedulers[node_kind]
+                action = actions[node_kind]
             else:
-                check_transition_handler = st.check_retry_transition
-                change_state_handler = change_state_handlers['retry']
-                scheduler = schedulers['retry']
-                all_retry_atoms.append(atom)
-            edge_deciders = {}
-            for previous_atom in execution_graph.predecessors(atom):
-                # If there is any link function that says if this connection
-                # is able to run (or should not) ensure we retain it and use
-                # it later as needed.
-                u_v_data = execution_graph.adj[previous_atom][atom]
-                u_v_decider = u_v_data.get(flow_type.LINK_DECIDER)
-                if u_v_decider is not None:
-                    edge_deciders[previous_atom.name] = u_v_decider
+                raise exc.CompilationFailure("Unknown node kind '%s'"
+                                             " encountered" % node_kind)
+            metadata = {}
+            deciders_it = self._walk_edge_deciders(graph, node)
+            walker = sc.ScopeWalker(self.compilation, node, names_only=True)
             metadata['scope_walker'] = walker
             metadata['check_transition_handler'] = check_transition_handler
             metadata['change_state_handler'] = change_state_handler
             metadata['scheduler'] = scheduler
-            metadata['edge_deciders'] = edge_deciders
-            self._atom_cache[atom.name] = metadata
-        self._atoms_by_kind['retry'] = all_retry_atoms
-        self._atoms_by_kind['task'] = all_task_atoms
+            metadata['edge_deciders'] = tuple(deciders_it)
+            metadata['action'] = action
+            self._atom_cache[node.name] = metadata
+        # TODO(harlowja): optimize the different decider depths to avoid
+        # repeated full successor searching; this can be done by searching
+        # for the widest depth of parent(s), and limiting the search of
+        # children by the that depth.
 
     @property
     def compilation(self):
@@ -108,13 +153,17 @@ class Runtime(object):
     def storage(self):
         return self._storage
 
+    @property
+    def options(self):
+        return self._options
+
     @misc.cachedproperty
     def analyzer(self):
         return an.Analyzer(self)
 
     @misc.cachedproperty
-    def runner(self):
-        return ru.Runner(self, async_utils.wait_for_any)
+    def builder(self):
+        return bu.MachineBuilder(self, waiters.wait_for_any)
 
     @misc.cachedproperty
     def completer(self):
@@ -144,13 +193,16 @@ class Runtime(object):
                              self._atom_notifier,
                              self._task_executor)
 
+    def _fetch_atom_metadata_entry(self, atom_name, metadata_key):
+        return self._atom_cache[atom_name][metadata_key]
+
     def check_atom_transition(self, atom, current_state, target_state):
         """Checks if the atom can transition to the provided target state."""
         # This does not check if the name exists (since this is only used
         # internally to the engine, and is not exposed to atoms that will
         # not exist and therefore doesn't need to handle that case).
-        metadata = self._atom_cache[atom.name]
-        check_transition_handler = metadata['check_transition_handler']
+        check_transition_handler = self._fetch_atom_metadata_entry(
+            atom.name, 'check_transition_handler')
         return check_transition_handler(current_state, target_state)
 
     def fetch_edge_deciders(self, atom):
@@ -158,30 +210,24 @@ class Runtime(object):
         # This does not check if the name exists (since this is only used
         # internally to the engine, and is not exposed to atoms that will
         # not exist and therefore doesn't need to handle that case).
-        metadata = self._atom_cache[atom.name]
-        return metadata['edge_deciders']
-
-    def fetch_atoms_by_kind(self, kind):
-        """Fetches all the atoms of a given kind.
-
-        NOTE(harlowja): Currently only ``task`` or ``retry`` are valid
-                        kinds of atoms (requesting other kinds will just
-                        return empty lists).
-        """
-        return self._atoms_by_kind.get(kind, [])
+        return self._fetch_atom_metadata_entry(atom.name, 'edge_deciders')
 
     def fetch_scheduler(self, atom):
         """Fetches the cached specific scheduler for the given atom."""
         # This does not check if the name exists (since this is only used
         # internally to the engine, and is not exposed to atoms that will
         # not exist and therefore doesn't need to handle that case).
+        return self._fetch_atom_metadata_entry(atom.name, 'scheduler')
+
+    def fetch_action(self, atom):
+        """Fetches the cached action handler for the given atom."""
         metadata = self._atom_cache[atom.name]
-        return metadata['scheduler']
+        return metadata['action']
 
     def fetch_scopes_for(self, atom_name):
         """Fetches a walker of the visible scopes for the given atom."""
         try:
-            metadata = self._atom_cache[atom_name]
+            return self._fetch_atom_metadata_entry(atom_name, 'scope_walker')
         except KeyError:
             # This signals to the caller that there is no walker for whatever
             # atom name was given that doesn't really have any associated atom
@@ -190,21 +236,19 @@ class Runtime(object):
             # atom and users can provide random names that do not actually
             # exist...
             return None
-        else:
-            return metadata['scope_walker']
 
     # Various helper methods used by the runtime components; not for public
     # consumption...
 
-    def reset_nodes(self, atoms, state=st.PENDING, intention=st.EXECUTE):
+    def reset_atoms(self, atoms, state=st.PENDING, intention=st.EXECUTE):
         """Resets all the provided atoms to the given state and intention."""
         tweaked = []
         for atom in atoms:
-            metadata = self._atom_cache[atom.name]
             if state or intention:
                 tweaked.append((atom, state, intention))
             if state:
-                change_state_handler = metadata['change_state_handler']
+                change_state_handler = self._fetch_atom_metadata_entry(
+                    atom.name, 'change_state_handler')
                 change_state_handler(atom, state)
             if intention:
                 self.storage.set_atom_intention(atom.name, intention)
@@ -212,16 +256,18 @@ class Runtime(object):
 
     def reset_all(self, state=st.PENDING, intention=st.EXECUTE):
         """Resets all atoms to the given state and intention."""
-        return self.reset_nodes(self.analyzer.iterate_all_nodes(),
+        return self.reset_atoms(self.analyzer.iterate_nodes(com.ATOMS),
                                 state=state, intention=intention)
 
     def reset_subgraph(self, atom, state=st.PENDING, intention=st.EXECUTE):
         """Resets a atoms subgraph to the given state and intention.
 
-        The subgraph is contained of all of the atoms successors.
+        The subgraph is contained of **all** of the atoms successors.
         """
-        return self.reset_nodes(self.analyzer.iterate_subgraph(atom),
-                                state=state, intention=intention)
+        execution_graph = self._compilation.execution_graph
+        atoms_it = tr.depth_first_iterate(execution_graph, atom,
+                                          tr.Direction.FORWARD)
+        return self.reset_atoms(atoms_it, state=state, intention=intention)
 
     def retry_subflow(self, retry):
         """Prepares a retrys + its subgraph for execution.
@@ -230,5 +276,6 @@ class Runtime(object):
         subgraph (its successors) to the ``PENDING`` state with an ``EXECUTE``
         intention.
         """
-        self.storage.set_atom_intention(retry.name, st.EXECUTE)
-        self.reset_subgraph(retry)
+        tweaked = self.reset_atoms([retry], state=None, intention=st.EXECUTE)
+        tweaked.extend(self.reset_subgraph(retry))
+        return tweaked
