@@ -15,8 +15,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""Implementation of a SQLAlchemy storage backend."""
-
 from __future__ import absolute_import
 
 import contextlib
@@ -25,6 +23,7 @@ import functools
 import time
 
 from oslo_utils import strutils
+import retrying
 import six
 import sqlalchemy as sa
 from sqlalchemy import exc as sa_exc
@@ -36,8 +35,7 @@ from taskflow import logging
 from taskflow.persistence.backends.sqlalchemy import migration
 from taskflow.persistence.backends.sqlalchemy import tables
 from taskflow.persistence import base
-from taskflow.persistence import logbook
-from taskflow.types import failure
+from taskflow.persistence import models
 from taskflow.utils import eventlet_utils
 from taskflow.utils import misc
 
@@ -110,6 +108,12 @@ DEFAULT_TXN_ISOLATION_LEVELS = {
 }
 
 
+def _log_statements(log_level, conn, cursor, statement, parameters, *args):
+    if LOG.isEnabledFor(log_level):
+        LOG.log(log_level, "Running statement '%s' with parameters %s",
+                statement, parameters)
+
+
 def _in_any(reason, err_haystack):
     """Checks if any elements of the haystack are in the given reason."""
     for err in err_haystack:
@@ -180,7 +184,7 @@ def _ping_listener(dbapi_conn, connection_rec, connection_proxy):
             raise
 
 
-class Alchemist(object):
+class _Alchemist(object):
     """Internal <-> external row <-> objects + other helper functions.
 
     NOTE(harlowja): for internal usage only.
@@ -190,37 +194,37 @@ class Alchemist(object):
 
     @staticmethod
     def convert_flow_detail(row):
-        return logbook.FlowDetail.from_dict(dict(row.items()))
+        return models.FlowDetail.from_dict(dict(row.items()))
 
     @staticmethod
     def convert_book(row):
-        return logbook.LogBook.from_dict(dict(row.items()))
+        return models.LogBook.from_dict(dict(row.items()))
 
     @staticmethod
     def convert_atom_detail(row):
         row = dict(row.items())
-        atom_cls = logbook.atom_detail_class(row.pop('atom_type'))
+        atom_cls = models.atom_detail_class(row.pop('atom_type'))
         return atom_cls.from_dict(row)
 
-    def _atom_query_iter(self, conn, parent_uuid):
+    def atom_query_iter(self, conn, parent_uuid):
         q = (sql.select([self._tables.atomdetails]).
              where(self._tables.atomdetails.c.parent_uuid == parent_uuid))
         for row in conn.execute(q):
             yield self.convert_atom_detail(row)
 
-    def _flow_query_iter(self, conn, parent_uuid):
+    def flow_query_iter(self, conn, parent_uuid):
         q = (sql.select([self._tables.flowdetails]).
              where(self._tables.flowdetails.c.parent_uuid == parent_uuid))
         for row in conn.execute(q):
             yield self.convert_flow_detail(row)
 
     def populate_book(self, conn, book):
-        for fd in self._flow_query_iter(conn, book.uuid):
+        for fd in self.flow_query_iter(conn, book.uuid):
             book.add(fd)
             self.populate_flow_detail(conn, fd)
 
     def populate_flow_detail(self, conn, fd):
-        for ad in self._atom_query_iter(conn, fd.uuid):
+        for ad in self.atom_query_iter(conn, fd.uuid):
             fd.add(ad)
 
 
@@ -239,14 +243,19 @@ class SQLAlchemyBackend(base.Backend):
             self._engine = engine
             self._owns_engine = False
         else:
-            self._engine = None
+            self._engine = self._create_engine(self._conf)
             self._owns_engine = True
         self._validated = False
+        try:
+            self._max_retries = misc.as_int(self._conf.get('max_retries'))
+        except TypeError:
+            self._max_retries = 0
 
-    def _create_engine(self):
+    @staticmethod
+    def _create_engine(conf):
         # NOTE(harlowja): copy the internal one so that we don't modify it via
         # all the popping that will happen below.
-        conf = copy.deepcopy(self._conf)
+        conf = copy.deepcopy(conf)
         engine_args = {
             'echo': _as_bool(conf.pop('echo', False)),
             'convert_unicode': _as_bool(conf.pop('convert_unicode', True)),
@@ -292,6 +301,13 @@ class SQLAlchemyBackend(base.Backend):
         # or engine arg overrides make sure we merge them in.
         engine_args.update(conf.pop('engine_args', {}))
         engine = sa.create_engine(sql_connection, **engine_args)
+        log_statements = conf.pop('log_statements', False)
+        if _as_bool(log_statements):
+            log_statements_level = conf.pop("log_statements_level",
+                                            logging.TRACE)
+            sa.event.listen(engine, "before_cursor_execute",
+                            functools.partial(_log_statements,
+                                              log_statements_level))
         checkin_yield = conf.pop('checkin_yield',
                                  eventlet_utils.EVENTLET_AVAILABLE)
         if _as_bool(checkin_yield):
@@ -309,31 +325,21 @@ class SQLAlchemyBackend(base.Backend):
 
     @property
     def engine(self):
-        if self._engine is None:
-            self._engine = self._create_engine()
         return self._engine
 
     def get_connection(self):
         conn = Connection(self)
         if not self._validated:
-            try:
-                max_retries = misc.as_int(self._conf.get('max_retries', None))
-            except TypeError:
-                max_retries = 0
-            conn.validate(max_retries=max_retries)
+            conn.validate(max_retries=self._max_retries)
             self._validated = True
         return conn
 
     def close(self):
-        if self._engine is not None and self._owns_engine:
-            # NOTE(harlowja): Only dispose of the engine and clear it from
-            # our local state if we actually own the engine in the first
-            # place. If the user passed in their own engine we should not
-            # be disposing it on their behalf (and we shouldn't be clearing
-            # our local engine either, since then we would just recreate a
-            # new engine if the engine property is accessed).
+        # NOTE(harlowja): Only dispose of the engine if we actually own the
+        # engine in the first place. If the user passed in their own engine
+        # we should not be disposing it on their behalf...
+        if self._owns_engine:
             self._engine.dispose()
-            self._engine = None
         self._validated = False
 
 
@@ -343,54 +349,48 @@ class Connection(base.Connection):
         self._engine = backend.engine
         self._metadata = sa.MetaData()
         self._tables = tables.fetch(self._metadata)
-        self._converter = Alchemist(self._tables)
+        self._converter = _Alchemist(self._tables)
 
     @property
     def backend(self):
         return self._backend
 
     def validate(self, max_retries=0):
+        """Performs basic **connection** validation of a sqlalchemy engine."""
 
-        def verify_connect(failures):
-            try:
-                # See if we can make a connection happen.
-                #
-                # NOTE(harlowja): note that even though we are connecting
-                # once it does not mean that we will be able to connect in
-                # the future, so this is more of a sanity test and is not
-                # complete connection insurance.
-                with contextlib.closing(self._engine.connect()):
-                    pass
-            except sa_exc.OperationalError as ex:
-                if _is_db_connection_error(six.text_type(ex.args[0])):
-                    failures.append(failure.Failure())
-                    return False
-            return True
+        def _retry_on_exception(exc):
+            LOG.warn("Engine connection (validate) failed due to '%s'", exc)
+            if isinstance(exc, sa_exc.OperationalError) and \
+               _is_db_connection_error(six.text_type(exc.args[0])):
+                # We may be able to fix this by retrying...
+                return True
+            if isinstance(exc, (sa_exc.TimeoutError,
+                                sa_exc.ResourceClosedError,
+                                sa_exc.DisconnectionError)):
+                # We may be able to fix this by retrying...
+                return True
+            # Other failures we likely can't fix by retrying...
+            return False
 
-        failures = []
-        if verify_connect(failures):
-            return
+        @retrying.retry(stop_max_attempt_number=max(0, int(max_retries)),
+                        # Ensure that the 2 ** retry number
+                        # is converted into milliseconds (thus why this
+                        # multiplies by 1000.0) because thats what retrying
+                        # lib. uses internally for whatever reason.
+                        wait_exponential_multiplier=1000.0,
+                        wrap_exception=False,
+                        retry_on_exception=_retry_on_exception)
+        def _try_connect(engine):
+            # See if we can make a connection happen.
+            #
+            # NOTE(harlowja): note that even though we are connecting
+            # once it does not mean that we will be able to connect in
+            # the future, so this is more of a sanity test and is not
+            # complete connection insurance.
+            with contextlib.closing(engine.connect()):
+                pass
 
-        # Sorry it didn't work out...
-        if max_retries <= 0:
-            failures[-1].reraise()
-
-        # Go through the exponential backoff loop and see if we can connect
-        # after a given number of backoffs (with a backoff sleeping period
-        # between each attempt)...
-        attempts_left = max_retries
-        for sleepy_secs in misc.ExponentialBackoff(max_retries):
-            LOG.warn("SQL connection failed due to '%s', %s attempts left.",
-                     failures[-1].exc, attempts_left)
-            LOG.info("Attempting to test the connection again in %s seconds.",
-                     sleepy_secs)
-            time.sleep(sleepy_secs)
-            if verify_connect(failures):
-                return
-            attempts_left -= 1
-
-        # Sorry it didn't work out...
-        failures[-1].reraise()
+        _try_connect(self._engine)
 
     def upgrade(self):
         try:
@@ -405,16 +405,18 @@ class Connection(base.Connection):
                     self._metadata.create_all(bind=conn)
                 else:
                     migration.db_sync(conn)
-        except sa_exc.SQLAlchemyError as e:
-            raise exc.StorageFailure("Failed upgrading database version", e)
+        except sa_exc.SQLAlchemyError:
+            exc.raise_with_cause(exc.StorageFailure,
+                                 "Failed upgrading database version")
 
     def clear_all(self):
         try:
             logbooks = self._tables.logbooks
             with self._engine.begin() as conn:
                 conn.execute(logbooks.delete())
-        except sa_exc.DBAPIError as e:
-            raise exc.StorageFailure("Failed clearing all entries", e)
+        except sa_exc.DBAPIError:
+            exc.raise_with_cause(exc.StorageFailure,
+                                 "Failed clearing all entries")
 
     def update_atom_details(self, atom_detail):
         try:
@@ -429,9 +431,10 @@ class Connection(base.Connection):
                 e_ad = self._converter.convert_atom_detail(row)
                 self._update_atom_details(conn, atom_detail, e_ad)
             return e_ad
-        except sa_exc.SQLAlchemyError as e:
-            raise exc.StorageFailure("Failed updating atom details with"
-                                     " uuid '%s'" % atom_detail.uuid, e)
+        except sa_exc.SQLAlchemyError:
+            exc.raise_with_cause(exc.StorageFailure,
+                                 "Failed updating atom details"
+                                 " with uuid '%s'" % atom_detail.uuid)
 
     def _insert_flow_details(self, conn, fd, parent_uuid):
         value = fd.to_dict()
@@ -443,7 +446,7 @@ class Connection(base.Connection):
     def _insert_atom_details(self, conn, ad, parent_uuid):
         value = ad.to_dict()
         value['parent_uuid'] = parent_uuid
-        value['atom_type'] = logbook.atom_detail_type(ad)
+        value['atom_type'] = models.atom_detail_type(ad)
         conn.execute(sql.insert(self._tables.atomdetails, value))
 
     def _update_atom_details(self, conn, ad, e_ad):
@@ -479,9 +482,10 @@ class Connection(base.Connection):
                 self._converter.populate_flow_detail(conn, e_fd)
                 self._update_flow_details(conn, flow_detail, e_fd)
             return e_fd
-        except sa_exc.SQLAlchemyError as e:
-            raise exc.StorageFailure("Failed updating flow details with"
-                                     " uuid '%s'" % flow_detail.uuid, e)
+        except sa_exc.SQLAlchemyError:
+            exc.raise_with_cause(exc.StorageFailure,
+                                 "Failed updating flow details with"
+                                 " uuid '%s'" % flow_detail.uuid)
 
     def destroy_logbook(self, book_uuid):
         try:
@@ -492,9 +496,9 @@ class Connection(base.Connection):
                 if r.rowcount == 0:
                     raise exc.NotFound("No logbook found with"
                                        " uuid '%s'" % book_uuid)
-        except sa_exc.DBAPIError as e:
-            raise exc.StorageFailure("Failed destroying"
-                                     " logbook '%s'" % book_uuid, e)
+        except sa_exc.DBAPIError:
+            exc.raise_with_cause(exc.StorageFailure,
+                                 "Failed destroying logbook '%s'" % book_uuid)
 
     def save_logbook(self, book):
         try:
@@ -523,11 +527,12 @@ class Connection(base.Connection):
                     for fd in book:
                         self._insert_flow_details(conn, fd, book.uuid)
                     return book
-        except sa_exc.DBAPIError as e:
-            raise exc.StorageFailure("Failed saving logbook"
-                                     " '%s'" % book.uuid, e)
+        except sa_exc.DBAPIError:
+            exc.raise_with_cause(
+                exc.StorageFailure,
+                "Failed saving logbook '%s'" % book.uuid)
 
-    def get_logbook(self, book_uuid):
+    def get_logbook(self, book_uuid, lazy=False):
         try:
             logbooks = self._tables.logbooks
             with contextlib.closing(self._engine.connect()) as conn:
@@ -538,25 +543,91 @@ class Connection(base.Connection):
                     raise exc.NotFound("No logbook found with"
                                        " uuid '%s'" % book_uuid)
                 book = self._converter.convert_book(row)
-                self._converter.populate_book(conn, book)
+                if not lazy:
+                    self._converter.populate_book(conn, book)
                 return book
-        except sa_exc.DBAPIError as e:
-            raise exc.StorageFailure(
-                "Failed getting logbook '%s'" % book_uuid, e)
+        except sa_exc.DBAPIError:
+            exc.raise_with_cause(exc.StorageFailure,
+                                 "Failed getting logbook '%s'" % book_uuid)
 
-    def get_logbooks(self):
+    def get_logbooks(self, lazy=False):
         gathered = []
         try:
             with contextlib.closing(self._engine.connect()) as conn:
                 q = sql.select([self._tables.logbooks])
                 for row in conn.execute(q):
                     book = self._converter.convert_book(row)
-                    self._converter.populate_book(conn, book)
+                    if not lazy:
+                        self._converter.populate_book(conn, book)
                     gathered.append(book)
-        except sa_exc.DBAPIError as e:
-            raise exc.StorageFailure("Failed getting logbooks", e)
+        except sa_exc.DBAPIError:
+            exc.raise_with_cause(exc.StorageFailure,
+                                 "Failed getting logbooks")
         for book in gathered:
             yield book
 
+    def get_flows_for_book(self, book_uuid, lazy=False):
+        gathered = []
+        try:
+            with contextlib.closing(self._engine.connect()) as conn:
+                for fd in self._converter.flow_query_iter(conn, book_uuid):
+                    if not lazy:
+                        self._converter.populate_flow_detail(conn, fd)
+                    gathered.append(fd)
+        except sa_exc.DBAPIError:
+            exc.raise_with_cause(exc.StorageFailure,
+                                 "Failed getting flow details in"
+                                 " logbook '%s'" % book_uuid)
+        for flow_details in gathered:
+            yield flow_details
+
+    def get_flow_details(self, fd_uuid, lazy=False):
+        try:
+            flowdetails = self._tables.flowdetails
+            with self._engine.begin() as conn:
+                q = (sql.select([flowdetails]).
+                     where(flowdetails.c.uuid == fd_uuid))
+                row = conn.execute(q).first()
+                if not row:
+                    raise exc.NotFound("No flow details found with uuid"
+                                       " '%s'" % fd_uuid)
+                fd = self._converter.convert_flow_detail(row)
+                if not lazy:
+                    self._converter.populate_flow_detail(conn, fd)
+                return fd
+        except sa_exc.SQLAlchemyError:
+            exc.raise_with_cause(exc.StorageFailure,
+                                 "Failed getting flow details with"
+                                 " uuid '%s'" % fd_uuid)
+
+    def get_atom_details(self, ad_uuid):
+        try:
+            atomdetails = self._tables.atomdetails
+            with self._engine.begin() as conn:
+                q = (sql.select([atomdetails]).
+                     where(atomdetails.c.uuid == ad_uuid))
+                row = conn.execute(q).first()
+                if not row:
+                    raise exc.NotFound("No atom details found with uuid"
+                                       " '%s'" % ad_uuid)
+                return self._converter.convert_atom_detail(row)
+        except sa_exc.SQLAlchemyError:
+            exc.raise_with_cause(exc.StorageFailure,
+                                 "Failed getting atom details with"
+                                 " uuid '%s'" % ad_uuid)
+
+    def get_atoms_for_flow(self, fd_uuid):
+        gathered = []
+        try:
+            with contextlib.closing(self._engine.connect()) as conn:
+                for ad in self._converter.atom_query_iter(conn, fd_uuid):
+                    gathered.append(ad)
+        except sa_exc.DBAPIError:
+            exc.raise_with_cause(exc.StorageFailure,
+                                 "Failed getting atom details in flow"
+                                 " detail '%s'" % fd_uuid)
+        for atom_details in gathered:
+            yield atom_details
+
     def close(self):
         pass
