@@ -19,7 +19,7 @@ import contextlib
 from oslo_utils import uuidutils
 
 from taskflow import exceptions as exc
-from taskflow.persistence import logbook
+from taskflow.persistence import models
 from taskflow import states
 from taskflow.types import failure
 
@@ -31,15 +31,15 @@ class PersistenceTestMixin(object):
     def test_task_detail_update_not_existing(self):
         lb_id = uuidutils.generate_uuid()
         lb_name = 'lb-%s' % (lb_id)
-        lb = logbook.LogBook(name=lb_name, uuid=lb_id)
-        fd = logbook.FlowDetail('test', uuid=uuidutils.generate_uuid())
+        lb = models.LogBook(name=lb_name, uuid=lb_id)
+        fd = models.FlowDetail('test', uuid=uuidutils.generate_uuid())
         lb.add(fd)
-        td = logbook.TaskDetail("detail-1", uuid=uuidutils.generate_uuid())
+        td = models.TaskDetail("detail-1", uuid=uuidutils.generate_uuid())
         fd.add(td)
         with contextlib.closing(self._get_connection()) as conn:
             conn.save_logbook(lb)
 
-        td2 = logbook.TaskDetail("detail-1", uuid=uuidutils.generate_uuid())
+        td2 = models.TaskDetail("detail-1", uuid=uuidutils.generate_uuid())
         fd.add(td2)
         with contextlib.closing(self._get_connection()) as conn:
             conn.update_flow_details(fd)
@@ -53,13 +53,13 @@ class PersistenceTestMixin(object):
     def test_flow_detail_update_not_existing(self):
         lb_id = uuidutils.generate_uuid()
         lb_name = 'lb-%s' % (lb_id)
-        lb = logbook.LogBook(name=lb_name, uuid=lb_id)
-        fd = logbook.FlowDetail('test', uuid=uuidutils.generate_uuid())
+        lb = models.LogBook(name=lb_name, uuid=lb_id)
+        fd = models.FlowDetail('test', uuid=uuidutils.generate_uuid())
         lb.add(fd)
         with contextlib.closing(self._get_connection()) as conn:
             conn.save_logbook(lb)
 
-        fd2 = logbook.FlowDetail('test-2', uuid=uuidutils.generate_uuid())
+        fd2 = models.FlowDetail('test-2', uuid=uuidutils.generate_uuid())
         lb.add(fd2)
         with contextlib.closing(self._get_connection()) as conn:
             conn.save_logbook(lb)
@@ -69,11 +69,32 @@ class PersistenceTestMixin(object):
         self.assertIsNotNone(lb2.find(fd.uuid))
         self.assertIsNotNone(lb2.find(fd2.uuid))
 
+    def test_logbook_save_retrieve_many(self):
+        lb_ids = {}
+        for i in range(0, 10):
+            lb_id = uuidutils.generate_uuid()
+            lb_name = 'lb-%s-%s' % (i, lb_id)
+            lb = models.LogBook(name=lb_name, uuid=lb_id)
+            lb_ids[lb_id] = True
+
+            # Should not already exist
+            with contextlib.closing(self._get_connection()) as conn:
+                self.assertRaises(exc.NotFound, conn.get_logbook, lb_id)
+                conn.save_logbook(lb)
+
+        # Now fetch them all
+        with contextlib.closing(self._get_connection()) as conn:
+            lbs = conn.get_logbooks()
+            for lb in lbs:
+                self.assertIn(lb.uuid, lb_ids)
+                lb_ids.pop(lb.uuid)
+            self.assertEqual(0, len(lb_ids))
+
     def test_logbook_save_retrieve(self):
         lb_id = uuidutils.generate_uuid()
         lb_meta = {'1': 2}
         lb_name = 'lb-%s' % (lb_id)
-        lb = logbook.LogBook(name=lb_name, uuid=lb_id)
+        lb = models.LogBook(name=lb_name, uuid=lb_id)
         lb.meta = lb_meta
 
         # Should not already exist
@@ -94,8 +115,8 @@ class PersistenceTestMixin(object):
     def test_flow_detail_save(self):
         lb_id = uuidutils.generate_uuid()
         lb_name = 'lb-%s' % (lb_id)
-        lb = logbook.LogBook(name=lb_name, uuid=lb_id)
-        fd = logbook.FlowDetail('test', uuid=uuidutils.generate_uuid())
+        lb = models.LogBook(name=lb_name, uuid=lb_id)
+        fd = models.FlowDetail('test', uuid=uuidutils.generate_uuid())
         lb.add(fd)
 
         # Ensure we can't save it since its owning logbook hasn't been
@@ -113,8 +134,8 @@ class PersistenceTestMixin(object):
     def test_flow_detail_meta_update(self):
         lb_id = uuidutils.generate_uuid()
         lb_name = 'lb-%s' % (lb_id)
-        lb = logbook.LogBook(name=lb_name, uuid=lb_id)
-        fd = logbook.FlowDetail('test', uuid=uuidutils.generate_uuid())
+        lb = models.LogBook(name=lb_name, uuid=lb_id)
+        fd = models.FlowDetail('test', uuid=uuidutils.generate_uuid())
         fd.meta = {'test': 42}
         lb.add(fd)
 
@@ -128,15 +149,31 @@ class PersistenceTestMixin(object):
         with contextlib.closing(self._get_connection()) as conn:
             lb2 = conn.get_logbook(lb_id)
         fd2 = lb2.find(fd.uuid)
-        self.assertEqual(fd2.meta.get('test'), 43)
+        self.assertEqual(43, fd2.meta.get('test'))
+
+    def test_flow_detail_lazy_fetch(self):
+        lb_id = uuidutils.generate_uuid()
+        lb_name = 'lb-%s' % (lb_id)
+        lb = models.LogBook(name=lb_name, uuid=lb_id)
+        fd = models.FlowDetail('test', uuid=uuidutils.generate_uuid())
+        td = models.TaskDetail("detail-1", uuid=uuidutils.generate_uuid())
+        td.version = '4.2'
+        fd.add(td)
+        lb.add(fd)
+        with contextlib.closing(self._get_connection()) as conn:
+            conn.save_logbook(lb)
+        with contextlib.closing(self._get_connection()) as conn:
+            fd2 = conn.get_flow_details(fd.uuid, lazy=True)
+            self.assertEqual(0, len(fd2))
+            self.assertEqual(1, len(fd))
 
     def test_task_detail_save(self):
         lb_id = uuidutils.generate_uuid()
         lb_name = 'lb-%s' % (lb_id)
-        lb = logbook.LogBook(name=lb_name, uuid=lb_id)
-        fd = logbook.FlowDetail('test', uuid=uuidutils.generate_uuid())
+        lb = models.LogBook(name=lb_name, uuid=lb_id)
+        fd = models.FlowDetail('test', uuid=uuidutils.generate_uuid())
         lb.add(fd)
-        td = logbook.TaskDetail("detail-1", uuid=uuidutils.generate_uuid())
+        td = models.TaskDetail("detail-1", uuid=uuidutils.generate_uuid())
         fd.add(td)
 
         # Ensure we can't save it since its owning logbook hasn't been
@@ -155,10 +192,10 @@ class PersistenceTestMixin(object):
     def test_task_detail_meta_update(self):
         lb_id = uuidutils.generate_uuid()
         lb_name = 'lb-%s' % (lb_id)
-        lb = logbook.LogBook(name=lb_name, uuid=lb_id)
-        fd = logbook.FlowDetail('test', uuid=uuidutils.generate_uuid())
+        lb = models.LogBook(name=lb_name, uuid=lb_id)
+        fd = models.FlowDetail('test', uuid=uuidutils.generate_uuid())
         lb.add(fd)
-        td = logbook.TaskDetail("detail-1", uuid=uuidutils.generate_uuid())
+        td = models.TaskDetail("detail-1", uuid=uuidutils.generate_uuid())
         td.meta = {'test': 42}
         fd.add(td)
 
@@ -175,16 +212,16 @@ class PersistenceTestMixin(object):
             lb2 = conn.get_logbook(lb_id)
         fd2 = lb2.find(fd.uuid)
         td2 = fd2.find(td.uuid)
-        self.assertEqual(td2.meta.get('test'), 43)
-        self.assertIsInstance(td2, logbook.TaskDetail)
+        self.assertEqual(43, td2.meta.get('test'))
+        self.assertIsInstance(td2, models.TaskDetail)
 
     def test_task_detail_with_failure(self):
         lb_id = uuidutils.generate_uuid()
         lb_name = 'lb-%s' % (lb_id)
-        lb = logbook.LogBook(name=lb_name, uuid=lb_id)
-        fd = logbook.FlowDetail('test', uuid=uuidutils.generate_uuid())
+        lb = models.LogBook(name=lb_name, uuid=lb_id)
+        fd = models.FlowDetail('test', uuid=uuidutils.generate_uuid())
         lb.add(fd)
-        td = logbook.TaskDetail("detail-1", uuid=uuidutils.generate_uuid())
+        td = models.TaskDetail("detail-1", uuid=uuidutils.generate_uuid())
 
         try:
             raise RuntimeError('Woot!')
@@ -203,21 +240,21 @@ class PersistenceTestMixin(object):
             lb2 = conn.get_logbook(lb_id)
         fd2 = lb2.find(fd.uuid)
         td2 = fd2.find(td.uuid)
-        self.assertEqual(td2.failure.exception_str, 'Woot!')
+        self.assertEqual('Woot!', td2.failure.exception_str)
         self.assertIs(td2.failure.check(RuntimeError), RuntimeError)
-        self.assertEqual(td2.failure.traceback_str, td.failure.traceback_str)
-        self.assertIsInstance(td2, logbook.TaskDetail)
+        self.assertEqual(td.failure.traceback_str, td2.failure.traceback_str)
+        self.assertIsInstance(td2, models.TaskDetail)
 
     def test_logbook_merge_flow_detail(self):
         lb_id = uuidutils.generate_uuid()
         lb_name = 'lb-%s' % (lb_id)
-        lb = logbook.LogBook(name=lb_name, uuid=lb_id)
-        fd = logbook.FlowDetail('test', uuid=uuidutils.generate_uuid())
+        lb = models.LogBook(name=lb_name, uuid=lb_id)
+        fd = models.FlowDetail('test', uuid=uuidutils.generate_uuid())
         lb.add(fd)
         with contextlib.closing(self._get_connection()) as conn:
             conn.save_logbook(lb)
-        lb2 = logbook.LogBook(name=lb_name, uuid=lb_id)
-        fd2 = logbook.FlowDetail('test2', uuid=uuidutils.generate_uuid())
+        lb2 = models.LogBook(name=lb_name, uuid=lb_id)
+        fd2 = models.FlowDetail('test2', uuid=uuidutils.generate_uuid())
         lb2.add(fd2)
         with contextlib.closing(self._get_connection()) as conn:
             conn.save_logbook(lb2)
@@ -228,8 +265,8 @@ class PersistenceTestMixin(object):
     def test_logbook_add_flow_detail(self):
         lb_id = uuidutils.generate_uuid()
         lb_name = 'lb-%s' % (lb_id)
-        lb = logbook.LogBook(name=lb_name, uuid=lb_id)
-        fd = logbook.FlowDetail('test', uuid=uuidutils.generate_uuid())
+        lb = models.LogBook(name=lb_name, uuid=lb_id)
+        fd = models.FlowDetail('test', uuid=uuidutils.generate_uuid())
         lb.add(fd)
         with contextlib.closing(self._get_connection()) as conn:
             conn.save_logbook(lb)
@@ -239,12 +276,25 @@ class PersistenceTestMixin(object):
             self.assertEqual(1, len(lb))
             self.assertEqual(fd.name, lb2.find(fd.uuid).name)
 
+    def test_logbook_lazy_fetch(self):
+        lb_id = uuidutils.generate_uuid()
+        lb_name = 'lb-%s' % (lb_id)
+        lb = models.LogBook(name=lb_name, uuid=lb_id)
+        fd = models.FlowDetail('test', uuid=uuidutils.generate_uuid())
+        lb.add(fd)
+        with contextlib.closing(self._get_connection()) as conn:
+            conn.save_logbook(lb)
+        with contextlib.closing(self._get_connection()) as conn:
+            lb2 = conn.get_logbook(lb_id, lazy=True)
+            self.assertEqual(0, len(lb2))
+            self.assertEqual(1, len(lb))
+
     def test_logbook_add_task_detail(self):
         lb_id = uuidutils.generate_uuid()
         lb_name = 'lb-%s' % (lb_id)
-        lb = logbook.LogBook(name=lb_name, uuid=lb_id)
-        fd = logbook.FlowDetail('test', uuid=uuidutils.generate_uuid())
-        td = logbook.TaskDetail("detail-1", uuid=uuidutils.generate_uuid())
+        lb = models.LogBook(name=lb_name, uuid=lb_id)
+        fd = models.FlowDetail('test', uuid=uuidutils.generate_uuid())
+        td = models.TaskDetail("detail-1", uuid=uuidutils.generate_uuid())
         td.version = '4.2'
         fd.add(td)
         lb.add(fd)
@@ -262,14 +312,14 @@ class PersistenceTestMixin(object):
             fd2 = lb2.find(fd.uuid)
             td2 = fd2.find(td.uuid)
             self.assertIsNot(td2, None)
-            self.assertEqual(td2.name, 'detail-1')
-            self.assertEqual(td2.version, '4.2')
-            self.assertEqual(td2.intention, states.EXECUTE)
+            self.assertEqual('detail-1', td2.name)
+            self.assertEqual('4.2', td2.version)
+            self.assertEqual(states.EXECUTE, td2.intention)
 
     def test_logbook_delete(self):
         lb_id = uuidutils.generate_uuid()
         lb_name = 'lb-%s' % (lb_id)
-        lb = logbook.LogBook(name=lb_name, uuid=lb_id)
+        lb = models.LogBook(name=lb_name, uuid=lb_id)
         with contextlib.closing(self._get_connection()) as conn:
             self.assertRaises(exc.NotFound, conn.destroy_logbook, lb_id)
         with contextlib.closing(self._get_connection()) as conn:
@@ -284,10 +334,10 @@ class PersistenceTestMixin(object):
     def test_task_detail_retry_type_(self):
         lb_id = uuidutils.generate_uuid()
         lb_name = 'lb-%s' % (lb_id)
-        lb = logbook.LogBook(name=lb_name, uuid=lb_id)
-        fd = logbook.FlowDetail('test', uuid=uuidutils.generate_uuid())
+        lb = models.LogBook(name=lb_name, uuid=lb_id)
+        fd = models.FlowDetail('test', uuid=uuidutils.generate_uuid())
         lb.add(fd)
-        rd = logbook.RetryDetail("detail-1", uuid=uuidutils.generate_uuid())
+        rd = models.RetryDetail("detail-1", uuid=uuidutils.generate_uuid())
         rd.intention = states.REVERT
         fd.add(rd)
 
@@ -300,16 +350,16 @@ class PersistenceTestMixin(object):
             lb2 = conn.get_logbook(lb_id)
         fd2 = lb2.find(fd.uuid)
         rd2 = fd2.find(rd.uuid)
-        self.assertEqual(rd2.intention, states.REVERT)
-        self.assertIsInstance(rd2, logbook.RetryDetail)
+        self.assertEqual(states.REVERT, rd2.intention)
+        self.assertIsInstance(rd2, models.RetryDetail)
 
     def test_retry_detail_save_with_task_failure(self):
         lb_id = uuidutils.generate_uuid()
         lb_name = 'lb-%s' % (lb_id)
-        lb = logbook.LogBook(name=lb_name, uuid=lb_id)
-        fd = logbook.FlowDetail('test', uuid=uuidutils.generate_uuid())
+        lb = models.LogBook(name=lb_name, uuid=lb_id)
+        fd = models.FlowDetail('test', uuid=uuidutils.generate_uuid())
         lb.add(fd)
-        rd = logbook.RetryDetail("retry-1", uuid=uuidutils.generate_uuid())
+        rd = models.RetryDetail("retry-1", uuid=uuidutils.generate_uuid())
         fail = failure.Failure.from_exception(RuntimeError('fail'))
         rd.results.append((42, {'some-task': fail}))
         fd.add(rd)
@@ -325,7 +375,7 @@ class PersistenceTestMixin(object):
             lb2 = conn.get_logbook(lb_id)
         fd2 = lb2.find(fd.uuid)
         rd2 = fd2.find(rd.uuid)
-        self.assertIsInstance(rd2, logbook.RetryDetail)
+        self.assertIsInstance(rd2, models.RetryDetail)
         fail2 = rd2.results[0][1].get('some-task')
         self.assertIsInstance(fail2, failure.Failure)
         self.assertTrue(fail.matches(fail2))
@@ -333,10 +383,10 @@ class PersistenceTestMixin(object):
     def test_retry_detail_save_intention(self):
         lb_id = uuidutils.generate_uuid()
         lb_name = 'lb-%s' % (lb_id)
-        lb = logbook.LogBook(name=lb_name, uuid=lb_id)
-        fd = logbook.FlowDetail('test', uuid=uuidutils.generate_uuid())
+        lb = models.LogBook(name=lb_name, uuid=lb_id)
+        fd = models.FlowDetail('test', uuid=uuidutils.generate_uuid())
         lb.add(fd)
-        rd = logbook.RetryDetail("retry-1", uuid=uuidutils.generate_uuid())
+        rd = models.RetryDetail("retry-1", uuid=uuidutils.generate_uuid())
         fd.add(rd)
 
         # save it
@@ -355,5 +405,5 @@ class PersistenceTestMixin(object):
             lb2 = conn.get_logbook(lb_id)
         fd2 = lb2.find(fd.uuid)
         rd2 = fd2.find(rd.uuid)
-        self.assertEqual(rd2.intention, states.REVERT)
-        self.assertIsInstance(rd2, logbook.RetryDetail)
+        self.assertEqual(states.REVERT, rd2.intention)
+        self.assertIsInstance(rd2, models.RetryDetail)
