@@ -42,6 +42,13 @@ from taskflow.utils import persistence_utils as p_utils
 from taskflow.utils import threading_utils as tu
 
 
+# Expected engine transitions when empty workflows are ran...
+_EMPTY_TRANSITIONS = [
+    states.RESUMING, states.SCHEDULING, states.WAITING,
+    states.ANALYZING, states.SUCCESS,
+]
+
+
 class EngineTaskTest(object):
 
     def test_run_task_as_flow(self):
@@ -72,14 +79,14 @@ class EngineTaskTest(object):
         with utils.CaptureListener(engine, values=values) as capturer:
             self.assertFailuresRegexp(RuntimeError, '^Woot', engine.run)
         self.assertEqual(expected, capturer.values)
-        self.assertEqual(engine.storage.get_flow_state(), states.REVERTED)
+        self.assertEqual(states.REVERTED, engine.storage.get_flow_state())
         with utils.CaptureListener(engine, values=values) as capturer:
             self.assertFailuresRegexp(RuntimeError, '^Woot', engine.run)
         now_expected = list(expected)
         now_expected.extend(['fail.t PENDING', 'fail.f PENDING'])
         now_expected.extend(expected)
         self.assertEqual(now_expected, values)
-        self.assertEqual(engine.storage.get_flow_state(), states.REVERTED)
+        self.assertEqual(states.REVERTED, engine.storage.get_flow_state())
 
     def test_invalid_flow_raises(self):
 
@@ -123,33 +130,33 @@ class EngineOptionalRequirementsTest(utils.EngineTestBase):
         engine = self._make_engine(flow_no_inject, store={'a': 3})
         engine.run()
         result = engine.storage.fetch_all()
-        self.assertEqual(result, {'a': 3, 'result': 15})
+        self.assertEqual({'a': 3, 'result': 15}, result)
 
         engine = self._make_engine(flow_no_inject,
                                    store={'a': 3, 'b': 7})
         engine.run()
         result = engine.storage.fetch_all()
-        self.assertEqual(result, {'a': 3, 'b': 7, 'result': 21})
+        self.assertEqual({'a': 3, 'b': 7, 'result': 21}, result)
 
         engine = self._make_engine(flow_inject_a, store={'a': 3})
         engine.run()
         result = engine.storage.fetch_all()
-        self.assertEqual(result, {'a': 3, 'result': 50})
+        self.assertEqual({'a': 3, 'result': 50}, result)
 
         engine = self._make_engine(flow_inject_a, store={'a': 3, 'b': 7})
         engine.run()
         result = engine.storage.fetch_all()
-        self.assertEqual(result, {'a': 3, 'b': 7, 'result': 70})
+        self.assertEqual({'a': 3, 'b': 7, 'result': 70}, result)
 
         engine = self._make_engine(flow_inject_b, store={'a': 3})
         engine.run()
         result = engine.storage.fetch_all()
-        self.assertEqual(result, {'a': 3, 'result': 3000})
+        self.assertEqual({'a': 3, 'result': 3000}, result)
 
         engine = self._make_engine(flow_inject_b, store={'a': 3, 'b': 7})
         engine.run()
         result = engine.storage.fetch_all()
-        self.assertEqual(result, {'a': 3, 'b': 7, 'result': 3000})
+        self.assertEqual({'a': 3, 'b': 7, 'result': 3000}, result)
 
 
 class EngineMultipleResultsTest(utils.EngineTestBase):
@@ -160,7 +167,7 @@ class EngineMultipleResultsTest(utils.EngineTestBase):
         engine = self._make_engine(flow)
         engine.run()
         result = engine.storage.fetch('x')
-        self.assertEqual(result, 1)
+        self.assertEqual(1, result)
 
     def test_many_results_visible_to(self):
         flow = lf.Flow("flow")
@@ -223,7 +230,7 @@ class EngineMultipleResultsTest(utils.EngineTestBase):
         engine = self._make_engine(flow, store={'x': 0})
         engine.run()
         result = engine.storage.fetch('x')
-        self.assertEqual(result, 0)
+        self.assertEqual(0, result)
 
     def test_fetch_all_with_a_single_result(self):
         flow = lf.Flow("flow")
@@ -232,7 +239,7 @@ class EngineMultipleResultsTest(utils.EngineTestBase):
         engine = self._make_engine(flow)
         engine.run()
         result = engine.storage.fetch_all()
-        self.assertEqual(result, {'x': 1})
+        self.assertEqual({'x': 1}, result)
 
     def test_fetch_all_with_two_results(self):
         flow = lf.Flow("flow")
@@ -241,7 +248,7 @@ class EngineMultipleResultsTest(utils.EngineTestBase):
         engine = self._make_engine(flow, store={'x': 0})
         engine.run()
         result = engine.storage.fetch_all()
-        self.assertEqual(result, {'x': [0, 1]})
+        self.assertEqual({'x': [0, 1]}, result)
 
     def test_task_can_update_value(self):
         flow = lf.Flow("flow")
@@ -250,15 +257,15 @@ class EngineMultipleResultsTest(utils.EngineTestBase):
         engine = self._make_engine(flow, store={'x': 0})
         engine.run()
         result = engine.storage.fetch_all()
-        self.assertEqual(result, {'x': [0, 1]})
+        self.assertEqual({'x': [0, 1]}, result)
 
 
 class EngineLinearFlowTest(utils.EngineTestBase):
 
-    def test_run_empty_flow(self):
+    def test_run_empty_linear_flow(self):
         flow = lf.Flow('flow-1')
         engine = self._make_engine(flow)
-        self.assertRaises(exc.Empty, engine.run)
+        self.assertEqual(_EMPTY_TRANSITIONS, list(engine.run_iter()))
 
     def test_overlap_parent_sibling_expected_result(self):
         flow = lf.Flow('flow-1')
@@ -315,7 +322,7 @@ class EngineLinearFlowTest(utils.EngineTestBase):
         expected = ['task1.t RUNNING', 'task1.t SUCCESS(5)',
                     'task2.t RUNNING', 'task2.t SUCCESS(5)']
         self.assertEqual(expected, capturer.values)
-        self.assertEqual(len(flow), 2)
+        self.assertEqual(2, len(flow))
 
     def test_sequential_flow_two_tasks_iter(self):
         flow = lf.Flow('flow-2').add(
@@ -329,7 +336,7 @@ class EngineLinearFlowTest(utils.EngineTestBase):
         expected = ['task1.t RUNNING', 'task1.t SUCCESS(5)',
                     'task2.t RUNNING', 'task2.t SUCCESS(5)']
         self.assertEqual(expected, capturer.values)
-        self.assertEqual(len(flow), 2)
+        self.assertEqual(2, len(flow))
 
     def test_sequential_flow_iter_suspend_resume(self):
         flow = lf.Flow('flow-2').add(
@@ -373,7 +380,7 @@ class EngineLinearFlowTest(utils.EngineTestBase):
         )
         engine = self._make_engine(flow)
         self.assertFailuresRegexp(RuntimeError, '^Woot', engine.run)
-        self.assertEqual(engine.storage.fetch_all(), {})
+        self.assertEqual({}, engine.storage.fetch_all())
 
     def test_revert_provided(self):
         flow = lf.Flow('revert').add(
@@ -382,7 +389,7 @@ class EngineLinearFlowTest(utils.EngineTestBase):
         )
         engine = self._make_engine(flow, store={'value': 0})
         self.assertFailuresRegexp(RuntimeError, '^Woot', engine.run)
-        self.assertEqual(engine.storage.get_revert_result('giver'), 2)
+        self.assertEqual(2, engine.storage.get_revert_result('giver'))
 
     def test_nasty_revert(self):
         flow = lf.Flow('revert').add(
@@ -456,10 +463,36 @@ class EngineLinearFlowTest(utils.EngineTestBase):
 
 class EngineParallelFlowTest(utils.EngineTestBase):
 
-    def test_run_empty_flow(self):
+    def test_run_empty_unordered_flow(self):
         flow = uf.Flow('p-1')
         engine = self._make_engine(flow)
-        self.assertRaises(exc.Empty, engine.run)
+        self.assertEqual(_EMPTY_TRANSITIONS, list(engine.run_iter()))
+
+    def test_parallel_flow_with_priority(self):
+        flow = uf.Flow('p-1')
+        for i in range(0, 10):
+            t = utils.ProgressingTask(name='task%s' % i)
+            t.priority = i
+            flow.add(t)
+        engine = self._make_engine(flow)
+        with utils.CaptureListener(engine, capture_flow=False) as capturer:
+            engine.run()
+        expected = [
+            'task9.t RUNNING',
+            'task8.t RUNNING',
+            'task7.t RUNNING',
+            'task6.t RUNNING',
+            'task5.t RUNNING',
+            'task4.t RUNNING',
+            'task3.t RUNNING',
+            'task2.t RUNNING',
+            'task1.t RUNNING',
+            'task0.t RUNNING',
+        ]
+        # NOTE(harlowja): chop off the gathering of SUCCESS states, since we
+        # don't care if thats in order...
+        gotten = capturer.values[0:10]
+        self.assertEqual(expected, gotten)
 
     def test_parallel_flow_one_task(self):
         flow = uf.Flow('p-1').add(
@@ -470,7 +503,7 @@ class EngineParallelFlowTest(utils.EngineTestBase):
             engine.run()
         expected = ['task1.t RUNNING', 'task1.t SUCCESS(5)']
         self.assertEqual(expected, capturer.values)
-        self.assertEqual(engine.storage.fetch_all(), {'a': 5})
+        self.assertEqual({'a': 5}, engine.storage.fetch_all())
 
     def test_parallel_flow_two_tasks(self):
         flow = uf.Flow('p-2').add(
@@ -533,8 +566,8 @@ class EngineParallelFlowTest(utils.EngineTestBase):
             engine.run()
         expected = ['task2.t RUNNING', 'task2.t SUCCESS(5)']
         self.assertEqual(expected, capturer.values)
-        self.assertEqual(engine.storage.fetch_all(),
-                         {'x1': 17, 'x2': 5})
+        self.assertEqual({'x1': 17, 'x2': 5},
+                         engine.storage.fetch_all())
 
 
 class EngineLinearAndUnorderedExceptionsTest(utils.EngineTestBase):
@@ -636,18 +669,106 @@ class EngineLinearAndUnorderedExceptionsTest(utils.EngineTestBase):
         self.assertNotIn('task2.t REVERTED(None)', capturer.values)
 
 
+class EngineDeciderDepthTest(utils.EngineTestBase):
+
+    def test_run_graph_flow_decider_various_depths(self):
+        sub_flow_1 = gf.Flow('g_1')
+        g_1_1 = utils.ProgressingTask(name='g_1-1')
+        sub_flow_1.add(g_1_1)
+        g_1 = utils.ProgressingTask(name='g-1')
+        g_2 = utils.ProgressingTask(name='g-2')
+        g_3 = utils.ProgressingTask(name='g-3')
+        g_4 = utils.ProgressingTask(name='g-4')
+        for a_depth, ran_how_many in [('all', 1),
+                                      ('atom', 4),
+                                      ('flow', 2),
+                                      ('neighbors', 3)]:
+            flow = gf.Flow('g')
+            flow.add(g_1, g_2, sub_flow_1, g_3, g_4)
+            flow.link(g_1, g_2,
+                      decider=lambda history: False,
+                      decider_depth=a_depth)
+            flow.link(g_2, sub_flow_1)
+            flow.link(g_2, g_3)
+            flow.link(g_3, g_4)
+            flow.link(g_1, sub_flow_1,
+                      decider=lambda history: True,
+                      decider_depth=a_depth)
+            e = self._make_engine(flow)
+            with utils.CaptureListener(e, capture_flow=False) as capturer:
+                e.run()
+            ran_tasks = 0
+            for outcome in capturer.values:
+                if outcome.endswith("RUNNING"):
+                    ran_tasks += 1
+            self.assertEqual(ran_how_many, ran_tasks)
+
+    def test_run_graph_flow_decider_jump_over_atom(self):
+        flow = gf.Flow('g')
+        a = utils.AddOneSameProvidesRequires("a", inject={'value': 0})
+        b = utils.AddOneSameProvidesRequires("b")
+        c = utils.AddOneSameProvidesRequires("c")
+        flow.add(a, b, c, resolve_requires=False)
+        flow.link(a, b, decider=lambda history: False,
+                  decider_depth='atom')
+        flow.link(b, c)
+        e = self._make_engine(flow)
+        e.run()
+        self.assertEqual(2, e.storage.get('c'))
+        self.assertEqual(states.IGNORE, e.storage.get_atom_state('b'))
+
+    def test_run_graph_flow_decider_jump_over_bad_atom(self):
+        flow = gf.Flow('g')
+        a = utils.NoopTask("a")
+        b = utils.FailingTask("b")
+        c = utils.NoopTask("c")
+        flow.add(a, b, c)
+        flow.link(a, b, decider=lambda history: False,
+                  decider_depth='atom')
+        flow.link(b, c)
+        e = self._make_engine(flow)
+        e.run()
+
+    def test_run_graph_flow_decider_revert(self):
+        flow = gf.Flow('g')
+        a = utils.NoopTask("a")
+        b = utils.NoopTask("b")
+        c = utils.FailingTask("c")
+        flow.add(a, b, c)
+        flow.link(a, b, decider=lambda history: False,
+                  decider_depth='atom')
+        flow.link(b, c)
+        e = self._make_engine(flow)
+        with utils.CaptureListener(e, capture_flow=False) as capturer:
+            # Wrapped failure here for WBE engine, make this better in
+            # the future, perhaps via a custom testtools matcher??
+            self.assertRaises((RuntimeError, exc.WrappedFailure), e.run)
+        expected = [
+            'a.t RUNNING',
+            'a.t SUCCESS(None)',
+            'b.t IGNORE',
+            'c.t RUNNING',
+            'c.t FAILURE(Failure: RuntimeError: Woot!)',
+            'c.t REVERTING',
+            'c.t REVERTED(None)',
+            'a.t REVERTING',
+            'a.t REVERTED(None)',
+        ]
+        self.assertEqual(expected, capturer.values)
+
+
 class EngineGraphFlowTest(utils.EngineTestBase):
 
-    def test_run_empty_flow(self):
+    def test_run_empty_graph_flow(self):
         flow = gf.Flow('g-1')
         engine = self._make_engine(flow)
-        self.assertRaises(exc.Empty, engine.run)
+        self.assertEqual(_EMPTY_TRANSITIONS, list(engine.run_iter()))
 
-    def test_run_nested_empty_flows(self):
+    def test_run_empty_nested_graph_flows(self):
         flow = gf.Flow('g-1').add(lf.Flow('l-1'),
                                   gf.Flow('g-2'))
         engine = self._make_engine(flow)
-        self.assertRaises(exc.Empty, engine.run)
+        self.assertEqual(_EMPTY_TRANSITIONS, list(engine.run_iter()))
 
     def test_graph_flow_one_task(self):
         flow = gf.Flow('g-1').add(
@@ -670,7 +791,7 @@ class EngineGraphFlowTest(utils.EngineTestBase):
         expected = set(['task2.t SUCCESS(5)', 'task2.t RUNNING',
                         'task1.t RUNNING', 'task1.t SUCCESS(5)'])
         self.assertEqual(expected, set(capturer.values))
-        self.assertEqual(len(flow), 2)
+        self.assertEqual(2, len(flow))
 
     def test_graph_flow_two_tasks(self):
         flow = gf.Flow('g-1-1').add(
@@ -728,7 +849,7 @@ class EngineGraphFlowTest(utils.EngineTestBase):
                     'task1.t REVERTING',
                     'task1.t REVERTED(None)']
         self.assertEqual(expected, capturer.values)
-        self.assertEqual(engine.storage.get_flow_state(), states.REVERTED)
+        self.assertEqual(states.REVERTED, engine.storage.get_flow_state())
 
     def test_graph_flow_four_tasks_revert_failure(self):
         flow = gf.Flow('g-3-nasty').add(
@@ -738,7 +859,7 @@ class EngineGraphFlowTest(utils.EngineTestBase):
 
         engine = self._make_engine(flow)
         self.assertFailuresRegexp(RuntimeError, '^Gotcha', engine.run)
-        self.assertEqual(engine.storage.get_flow_state(), states.FAILURE)
+        self.assertEqual(states.FAILURE, engine.storage.get_flow_state())
 
     def test_graph_flow_with_multireturn_and_multiargs_tasks(self):
         flow = gf.Flow('g-3-multi').add(
@@ -751,14 +872,14 @@ class EngineGraphFlowTest(utils.EngineTestBase):
         engine = self._make_engine(flow)
         engine.storage.inject({'x': 30})
         engine.run()
-        self.assertEqual(engine.storage.fetch_all(), {
+        self.assertEqual({
             'a': 1,
             'b': 3,
             'c': 5,
             'x': 30,
             'y': 38,
             'z': 42
-        })
+        }, engine.storage.fetch_all())
 
     def test_task_graph_property(self):
         flow = gf.Flow('test').add(
@@ -933,6 +1054,88 @@ class EngineResetTests(utils.EngineTestBase):
 
 class EngineGraphConditionalFlowTest(utils.EngineTestBase):
 
+    def test_graph_flow_conditional_jumps_across_2(self):
+        histories = []
+
+        def should_go(history):
+            histories.append(history)
+            return False
+
+        task1 = utils.ProgressingTask(name='task1')
+        task2 = utils.ProgressingTask(name='task2')
+        task3 = utils.ProgressingTask(name='task3')
+        task4 = utils.ProgressingTask(name='task4')
+
+        subflow = lf.Flow("more-work")
+        subsub_flow = lf.Flow("more-more-work")
+        subsub_flow.add(task3, task4)
+        subflow.add(subsub_flow)
+
+        flow = gf.Flow("main-work")
+        flow.add(task1, task2)
+        flow.link(task1, task2)
+        flow.add(subflow)
+        flow.link(task2, subflow, decider=should_go)
+
+        engine = self._make_engine(flow)
+        with utils.CaptureListener(engine, capture_flow=False) as capturer:
+            engine.run()
+
+        expected = [
+            'task1.t RUNNING',
+            'task1.t SUCCESS(5)',
+
+            'task2.t RUNNING',
+            'task2.t SUCCESS(5)',
+
+            'task3.t IGNORE',
+            'task4.t IGNORE',
+        ]
+        self.assertEqual(expected, capturer.values)
+        self.assertEqual(1, len(histories))
+        self.assertIn('task2', histories[0])
+
+    def test_graph_flow_conditional_jumps_across(self):
+        histories = []
+
+        def should_go(history):
+            histories.append(history)
+            return False
+
+        task1 = utils.ProgressingTask(name='task1')
+        task2 = utils.ProgressingTask(name='task2')
+        task3 = utils.ProgressingTask(name='task3')
+        task4 = utils.ProgressingTask(name='task4')
+
+        subflow = lf.Flow("more-work")
+        subflow.add(task3, task4)
+        flow = gf.Flow("main-work")
+        flow.add(task1, task2)
+        flow.link(task1, task2)
+        flow.add(subflow)
+        flow.link(task2, subflow, decider=should_go)
+        flow.link(task1, subflow, decider=should_go)
+
+        engine = self._make_engine(flow)
+        with utils.CaptureListener(engine, capture_flow=False) as capturer:
+            engine.run()
+
+        expected = [
+            'task1.t RUNNING',
+            'task1.t SUCCESS(5)',
+
+            'task2.t RUNNING',
+            'task2.t SUCCESS(5)',
+
+            'task3.t IGNORE',
+            'task4.t IGNORE',
+        ]
+        self.assertEqual(expected, capturer.values)
+        self.assertEqual(2, len(histories))
+        for i in range(0, 2):
+            self.assertIn('task1', histories[i])
+            self.assertIn('task2', histories[i])
+
     def test_graph_flow_conditional(self):
         flow = gf.Flow('root')
 
@@ -1110,11 +1313,11 @@ class EngineCheckingTaskTest(utils.EngineTestBase):
                 return 'RESULT'
 
             def revert(m_self, result, flow_failures):
-                self.assertEqual(result, 'RESULT')
-                self.assertEqual(list(flow_failures.keys()), ['fail1'])
+                self.assertEqual('RESULT', result)
+                self.assertEqual(['fail1'], list(flow_failures.keys()))
                 fail = flow_failures['fail1']
                 self.assertIsInstance(fail, failure.Failure)
-                self.assertEqual(str(fail), 'Failure: RuntimeError: Woot!')
+                self.assertEqual('Failure: RuntimeError: Woot!', str(fail))
 
         flow = lf.Flow('test').add(
             CheckingTask(),
@@ -1135,14 +1338,15 @@ class SerialEngineTest(EngineTaskTest,
                        EngineResetTests,
                        EngineGraphConditionalFlowTest,
                        EngineCheckingTaskTest,
+                       EngineDeciderDepthTest,
                        test.TestCase):
     def _make_engine(self, flow,
-                     flow_detail=None, store=None):
+                     flow_detail=None, store=None, **kwargs):
         return taskflow.engines.load(flow,
                                      flow_detail=flow_detail,
                                      engine='serial',
                                      backend=self.backend,
-                                     store=store)
+                                     store=store, **kwargs)
 
     def test_correct_load(self):
         engine = self._make_engine(utils.TaskNoRequiresNoReturns)
@@ -1164,11 +1368,13 @@ class ParallelEngineWithThreadsTest(EngineTaskTest,
                                     EngineMissingDepsTest,
                                     EngineGraphConditionalFlowTest,
                                     EngineCheckingTaskTest,
+                                    EngineDeciderDepthTest,
                                     test.TestCase):
     _EXECUTOR_WORKERS = 2
 
     def _make_engine(self, flow,
-                     flow_detail=None, executor=None, store=None):
+                     flow_detail=None, executor=None, store=None,
+                     **kwargs):
         if executor is None:
             executor = 'threads'
         return taskflow.engines.load(flow, flow_detail=flow_detail,
@@ -1176,7 +1382,8 @@ class ParallelEngineWithThreadsTest(EngineTaskTest,
                                      executor=executor,
                                      engine='parallel',
                                      store=store,
-                                     max_workers=self._EXECUTOR_WORKERS)
+                                     max_workers=self._EXECUTOR_WORKERS,
+                                     **kwargs)
 
     def test_correct_load(self):
         engine = self._make_engine(utils.TaskNoRequiresNoReturns)
@@ -1205,17 +1412,19 @@ class ParallelEngineWithEventletTest(EngineTaskTest,
                                      EngineMissingDepsTest,
                                      EngineGraphConditionalFlowTest,
                                      EngineCheckingTaskTest,
+                                     EngineDeciderDepthTest,
                                      test.TestCase):
 
     def _make_engine(self, flow,
-                     flow_detail=None, executor=None, store=None):
+                     flow_detail=None, executor=None, store=None,
+                     **kwargs):
         if executor is None:
             executor = futurist.GreenThreadPoolExecutor()
             self.addCleanup(executor.shutdown)
         return taskflow.engines.load(flow, flow_detail=flow_detail,
                                      backend=self.backend, engine='parallel',
                                      executor=executor,
-                                     store=store)
+                                     store=store, **kwargs)
 
 
 class ParallelEngineWithProcessTest(EngineTaskTest,
@@ -1228,6 +1437,7 @@ class ParallelEngineWithProcessTest(EngineTaskTest,
                                     EngineResetTests,
                                     EngineMissingDepsTest,
                                     EngineGraphConditionalFlowTest,
+                                    EngineDeciderDepthTest,
                                     test.TestCase):
     _EXECUTOR_WORKERS = 2
 
@@ -1236,7 +1446,8 @@ class ParallelEngineWithProcessTest(EngineTaskTest,
         self.assertIsInstance(engine, eng.ParallelActionEngine)
 
     def _make_engine(self, flow,
-                     flow_detail=None, executor=None, store=None):
+                     flow_detail=None, executor=None, store=None,
+                     **kwargs):
         if executor is None:
             executor = 'processes'
         return taskflow.engines.load(flow, flow_detail=flow_detail,
@@ -1244,7 +1455,8 @@ class ParallelEngineWithProcessTest(EngineTaskTest,
                                      engine='parallel',
                                      executor=executor,
                                      store=store,
-                                     max_workers=self._EXECUTOR_WORKERS)
+                                     max_workers=self._EXECUTOR_WORKERS,
+                                     **kwargs)
 
 
 class WorkerBasedEngineTest(EngineTaskTest,
@@ -1257,6 +1469,7 @@ class WorkerBasedEngineTest(EngineTaskTest,
                             EngineResetTests,
                             EngineMissingDepsTest,
                             EngineGraphConditionalFlowTest,
+                            EngineDeciderDepthTest,
                             test.TestCase):
     def setUp(self):
         super(WorkerBasedEngineTest, self).setUp()
@@ -1292,19 +1505,21 @@ class WorkerBasedEngineTest(EngineTaskTest,
         self.worker_thread = tu.daemon_thread(self.worker.run)
         self.worker_thread.start()
 
+        # Ensure worker and thread is stopped when test is done; these are
+        # called in reverse order, so make sure we signal the stop before
+        # performing the join (because the reverse won't work).
+        self.addCleanup(self.worker_thread.join)
+        self.addCleanup(self.worker.stop)
+
         # Make sure the worker is started before we can continue...
         self.worker.wait()
 
-    def tearDown(self):
-        self.worker.stop()
-        self.worker_thread.join()
-        super(WorkerBasedEngineTest, self).tearDown()
-
     def _make_engine(self, flow,
-                     flow_detail=None, store=None):
+                     flow_detail=None, store=None, **kwargs):
+        kwargs.update(self.engine_conf)
         return taskflow.engines.load(flow, flow_detail=flow_detail,
                                      backend=self.backend,
-                                     store=store, **self.engine_conf)
+                                     store=store, **kwargs)
 
     def test_correct_load(self):
         engine = self._make_engine(utils.TaskNoRequiresNoReturns)
