@@ -179,19 +179,19 @@ class TestSequenceMinus(test.TestCase):
 
     def test_simple_case(self):
         result = misc.sequence_minus([1, 2, 3, 4], [2, 3])
-        self.assertEqual(result, [1, 4])
+        self.assertEqual([1, 4], result)
 
     def test_subtrahend_has_extra_elements(self):
         result = misc.sequence_minus([1, 2, 3, 4], [2, 3, 5, 7, 13])
-        self.assertEqual(result, [1, 4])
+        self.assertEqual([1, 4], result)
 
     def test_some_items_are_equal(self):
         result = misc.sequence_minus([1, 1, 1, 1], [1, 1, 3])
-        self.assertEqual(result, [1, 1])
+        self.assertEqual([1, 1], result)
 
     def test_equal_items_not_continious(self):
         result = misc.sequence_minus([1, 2, 3, 1], [1, 3])
-        self.assertEqual(result, [2, 1])
+        self.assertEqual([2, 1], result)
 
 
 class TestReversedEnumerate(testscenarios.TestWithScenarios, test.TestCase):
@@ -244,24 +244,6 @@ class TestCountdownIter(test.TestCase):
         self.assertRaises(ValueError, six.next, it)
 
 
-class TestLookFor(test.TestCase):
-    def test_no_matches(self):
-        hay = [9, 10, 11]
-        self.assertEqual([], misc.look_for(hay, [1, 2, 3]))
-
-    def test_match_order(self):
-        hay = [6, 5, 4, 3, 2, 1]
-        priors = []
-        for i in range(0, 6):
-            priors.append(i + 1)
-            matches = misc.look_for(hay, priors)
-            self.assertGreater(0, len(matches))
-            self.assertIsSuperAndSubsequence(hay, matches)
-        hay = [10, 1, 15, 3, 5, 8, 44]
-        self.assertEqual([1, 15], misc.look_for(hay, [15, 1]))
-        self.assertEqual([10, 44], misc.look_for(hay, [44, 10]))
-
-
 class TestMergeUri(test.TestCase):
     def test_merge(self):
         url = "http://www.yahoo.com/?a=b&c=d"
@@ -301,11 +283,11 @@ class TestMergeUri(test.TestCase):
 class TestClamping(test.TestCase):
     def test_simple_clamp(self):
         result = misc.clamp(1.0, 2.0, 3.0)
-        self.assertEqual(result, 2.0)
+        self.assertEqual(2.0, result)
         result = misc.clamp(4.0, 2.0, 3.0)
-        self.assertEqual(result, 3.0)
+        self.assertEqual(3.0, result)
         result = misc.clamp(3.0, 4.0, 4.0)
-        self.assertEqual(result, 4.0)
+        self.assertEqual(4.0, result)
 
     def test_invalid_clamp(self):
         self.assertRaises(ValueError, misc.clamp, 0.0, 2.0, 1.0)
@@ -340,3 +322,34 @@ class TestIterable(test.TestCase):
 
     def test_dict(self):
         self.assertTrue(misc.is_iterable(dict()))
+
+
+class TestSafeCopyDict(testscenarios.TestWithScenarios):
+    scenarios = [
+        ('none', {'original': None, 'expected': {}}),
+        ('empty_dict', {'original': {}, 'expected': {}}),
+        ('empty_list', {'original': [], 'expected': {}}),
+        ('dict', {'original': {'a': 1, 'b': 2}, 'expected': {'a': 1, 'b': 2}}),
+    ]
+
+    def test_expected(self):
+        self.assertEqual(self.expected, misc.safe_copy_dict(self.original))
+        self.assertFalse(self.expected is misc.safe_copy_dict(self.original))
+
+    def test_mutated_post_copy(self):
+        a = {"a": "b"}
+        a_2 = misc.safe_copy_dict(a)
+        a['a'] = 'c'
+        self.assertEqual("b", a_2['a'])
+        self.assertEqual("c", a['a'])
+
+
+class TestSafeCopyDictRaises(testscenarios.TestWithScenarios):
+    scenarios = [
+        ('list', {'original': [1, 2], 'exception': TypeError}),
+        ('tuple', {'original': (1, 2), 'exception': TypeError}),
+        ('set', {'original': set([1, 2]), 'exception': TypeError}),
+    ]
+
+    def test_exceptions(self):
+        self.assertRaises(self.exception, misc.safe_copy_dict, self.original)
