@@ -14,6 +14,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections
 import copy
 import os
 import sys
@@ -24,9 +25,11 @@ from oslo_utils import reflection
 import six
 
 from taskflow import exceptions as exc
+from taskflow.utils import iter_utils
 from taskflow.utils import mixins
 from taskflow.utils import schema_utils as su
 
+
 _exception_message = encodeutils.exception_to_unicode
 
 
@@ -40,23 +43,6 @@ def _copy_exc_info(exc_info):
     return (exc_type, copy.copy(exc_value), tb)
 
 
-def _fill_iter(it, desired_len, filler=None):
-    """Iterates over a provided iterator up to the desired length.
-
-    If the source iterator does not have enough values then the filler
-    value is yielded until the desired length is reached.
-    """
-    count = 0
-    for value in it:
-        if count >= desired_len:
-            return
-        yield value
-        count += 1
-    while count < desired_len:
-        yield filler
-        count += 1
-
-
 def _are_equal_exc_info_tuples(ei1, ei2):
     if ei1 == ei2:
         return True
@@ -86,7 +72,7 @@ class Failure(mixins.StrMixin):
     re-used later to re-raise, inspect, examine, log, print, serialize,
     deserialize...
 
-    One example where they are dependened upon is in the WBE engine. When a
+    One example where they are depended upon is in the WBE engine. When a
     remote worker throws an exception, the WBE based engine will receive that
     exception and desire to reraise it to the user/caller of the WBE based
     engine for appropriate handling (this matches the behavior of non-remote
@@ -137,6 +123,13 @@ class Failure(mixins.StrMixin):
     """
     DICT_VERSION = 1
 
+    BASE_EXCEPTIONS = ('BaseException', 'Exception')
+    """
+    Root exceptions of all other python exceptions.
+
+    See: https://docs.python.org/2/library/exceptions.html
+    """
+
     #: Expected failure schema (in json schema format).
     SCHEMA = {
         "$ref": "#/definitions/cause",
@@ -148,6 +141,10 @@ class Failure(mixins.StrMixin):
                         "type": "integer",
                         "minimum": 0,
                     },
+                    'exc_args': {
+                        "type": "array",
+                        "minItems": 0,
+                    },
                     'exception_str': {
                         "type": "string",
                     },
@@ -190,6 +187,7 @@ class Failure(mixins.StrMixin):
                     raise ValueError("Provided 'exc_info' must contain three"
                                      " elements")
             self._exc_info = exc_info
+            self._exc_args = tuple(getattr(exc_info[1], 'args', []))
             self._exc_type_names = tuple(
                 reflection.get_all_class_names(exc_info[0], up_to=Exception))
             if not self._exc_type_names:
@@ -202,6 +200,7 @@ class Failure(mixins.StrMixin):
         else:
             self._causes = kwargs.pop('causes', None)
             self._exc_info = exc_info
+            self._exc_args = tuple(kwargs.pop('exc_args', []))
             self._exception_str = kwargs.pop('exception_str')
             self._exc_type_names = tuple(kwargs.pop('exc_type_names', []))
             self._traceback_str = kwargs.pop('traceback_str', None)
@@ -222,16 +221,35 @@ class Failure(mixins.StrMixin):
 
     @classmethod
     def validate(cls, data):
+        """Validate input data matches expected failure ``dict`` format."""
         try:
             su.schema_validate(data, cls.SCHEMA)
         except su.ValidationError as e:
             raise exc.InvalidFormat("Failure data not of the"
                                     " expected format: %s" % (e.message), e)
+        else:
+            # Ensure that all 'exc_type_names' originate from one of
+            # BASE_EXCEPTIONS, because those are the root exceptions that
+            # python mandates/provides and anything else is invalid...
+            causes = collections.deque([data])
+            while causes:
+                cause = causes.popleft()
+                root_exc_type = cause['exc_type_names'][-1]
+                if root_exc_type not in cls.BASE_EXCEPTIONS:
+                    raise exc.InvalidFormat(
+                        "Failure data 'exc_type_names' must"
+                        " have an initial exception type that is one"
+                        " of %s types: '%s' is not one of those"
+                        " types" % (cls.BASE_EXCEPTIONS, root_exc_type))
+                sub_causes = cause.get('causes')
+                if sub_causes:
+                    causes.extend(sub_causes)
 
     def _matches(self, other):
         if self is other:
             return True
         return (self._exc_type_names == other._exc_type_names
+                and self.exception_args == other.exception_args
                 and self.exception_str == other.exception_str
                 and self.traceback_str == other.traceback_str
                 and self.causes == other.causes)
@@ -267,7 +285,7 @@ class Failure(mixins.StrMixin):
 
     @property
     def exception(self):
-        """Exception value, or None if exception value is not present.
+        """Exception value, or none if exception value is not present.
 
         Exception value may be lost during serialization.
         """
@@ -281,9 +299,19 @@ class Failure(mixins.StrMixin):
         """String representation of exception."""
         return self._exception_str
 
+    @property
+    def exception_args(self):
+        """Tuple of arguments given to the exception constructor."""
+        return self._exc_args
+
     @property
     def exc_info(self):
-        """Exception info tuple or None."""
+        """Exception info tuple or none.
+
+        See: https://docs.python.org/2/library/sys.html#sys.exc_info for what
+             the contents of this tuple are (if none, then no contents can
+             be examined).
+        """
         return self._exc_info
 
     @property
@@ -433,6 +461,12 @@ class Failure(mixins.StrMixin):
 
     def __setstate__(self, dct):
         self._exception_str = dct['exception_str']
+        if 'exc_args' in dct:
+            self._exc_args = tuple(dct['exc_args'])
+        else:
+            # Guess we got an older version somehow, before this
+            # was added, so at that point just set to an empty tuple...
+            self._exc_args = ()
         self._traceback_str = dct['traceback_str']
         self._exc_type_names = dct['exc_type_names']
         if 'exc_info' in dct:
@@ -444,7 +478,7 @@ class Failure(mixins.StrMixin):
             # what the twisted people have done, see for example
             # twisted-13.0.0/twisted/python/failure.py#L89 for how they
             # created a fake traceback object...
-            self._exc_info = tuple(_fill_iter(dct['exc_info'], 3))
+            self._exc_info = tuple(iter_utils.fill(dct['exc_info'], 3))
         else:
             self._exc_info = None
         causes = dct.get('causes')
@@ -465,13 +499,18 @@ class Failure(mixins.StrMixin):
             data['causes'] = tuple(cls.from_dict(d) for d in causes)
         return cls(**data)
 
-    def to_dict(self):
-        """Converts this object to a dictionary."""
+    def to_dict(self, include_args=True):
+        """Converts this object to a dictionary.
+
+        :param include_args: boolean indicating whether to include the
+                             exception args in the output.
+        """
         return {
             'exception_str': self.exception_str,
             'traceback_str': self.traceback_str,
             'exc_type_names': list(self),
             'version': self.DICT_VERSION,
+            'exc_args': self.exception_args if include_args else tuple(),
             'causes': [f.to_dict() for f in self.causes],
         }
 
@@ -480,5 +519,6 @@ class Failure(mixins.StrMixin):
         return Failure(exc_info=_copy_exc_info(self.exc_info),
                        exception_str=self.exception_str,
                        traceback_str=self.traceback_str,
+                       exc_args=self.exception_args,
                        exc_type_names=self._exc_type_names[:],
                        causes=self._causes)
