@@ -15,6 +15,7 @@
 #    under the License.
 
 import collections
+import multiprocessing
 import threading
 
 import six
@@ -35,6 +36,17 @@ def get_ident():
     return _thread.get_ident()
 
 
+def get_optimal_thread_count(default=2):
+    """Try to guess optimal thread count for current system."""
+    try:
+        return multiprocessing.cpu_count() + 1
+    except NotImplementedError:
+        # NOTE(harlowja): apparently may raise so in this case we will
+        # just setup two threads since it's hard to know what else we
+        # should do in this situation.
+        return default
+
+
 def daemon_thread(target, *args, **kwargs):
     """Makes a daemon thread that calls the given target when started."""
     thread = threading.Thread(target=target, args=args, kwargs=kwargs)
